<?php

    // use the phpseclib ssh client
    set_include_path(get_include_path() . PATH_SEPARATOR . dirname(__FILE__) ."/../../phpseclib");
    require_once('Net/SFTP.php');
    require_once('Crypt/RSA.php');

    class Send_File_Action extends Reactor_Action
    {
        protected $tpl = 'send_file.php';
        protected $name = 'Upload a file';
		
		public static $registration = array(
			ACTION_NAME => 'Upload a file',
			ACTION_CATEGORY => 'Network',
			ACTION_DESCRIPTION => 'Uploads a file using either FTP or SFTP.',
			ACTION_AUTHOR => 'Jeremy Yunis, Nagios Enterprises, LLC',
			ACTION_VERSION => '1.0',
			ACTION_DATE => '05/15/2013', //Last version update
			ACTION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
			ACTION_TYPE => 'core', //core | user. core modules can't be deleted
			ACTION_META => array(),	//Misc information	
		); 		
		
        protected $fields = array(
            'hostname' => array(
                'label' => 'remote host',
                'validate' => 'trim|required|xss_clean|callback_check_hostname|prep_for_form',
            ),
            'username' => array(
                'label' => 'remote user',
                'validate' => 'trim|required|alpha_dash|xss_clean|prep_for_form',
            ),
            'ftp_type' => array(
                'label' => 'FTP type',
                'validate' => 'trim|required',
            ),
            'auth_type' => array(
                'label' => 'authentication type',
                'validate' => 'trim|required|xss_clean'
            ),
            'password' => array(
                'label' => 'remote password',
                'validate' => 'trim|xss_clean|prep_for_form|callback_check_password',
            ),
            'ssh_key' => array(
                'label' => 'SSH key',
                'validate' => 'trim|xss_clean|prep_for_form|callback_check_ssh_key',
            ),
            'ssh_key_pw' => array(
                'label' => 'SSH key password',
                'validate' => 'trim|xss_clean|callback_check_ssh_key_pw',
            ),
            'local_file' => array(
                'label' => 'local file',
                'validate' => 'trim|required|xss_clean|callback_check_filename',
            ),
            'remote_file' => array(
                'label' => 'remote file',
                'validate' => 'trim|required|xss_clean',
            ),
            'passive' => array(
                'label' => 'ftp mode',
                'validate' => 'trim|required'
            )
        );

        /** 
         * Form validation function to check a hostname.
         **/
        public function check_hostname($str)
        {
            $hostname_expression = '/^(?=.{1,255}$)[0-9A-Za-z]'.
                                   '(?:(?:[0-9A-Za-z]|\b-){0,61}[0-9A-Za-z])?'.
                                   '(?:\.[0-9A-Za-z](?:(?:[0-9A-Za-z]|\b-){0,61}[0-9A-Za-z])?)*\.?$/i';
            if(!preg_match($hostname_expression, $str)) {
                $this->ci->form_validation->set_message('check_hostname', 'The hostname specified is invalid.');
                return FALSE;
            } else {
                return TRUE;
            }
        }

        /**
         * Form validation function which requires a password to be entered if that auth type is selected.
         **/
        public function check_password($str)
        {
            $type = $this->ci->input->post('auth_type');
            if($type == 'password') {
                if(!strlen($str)) {
                    $this->ci->form_validation->set_message('check_password', 'This field is required for password authentication.');
                    return FALSE;
                }
            }
            return TRUE;
        }
        /**
         * Form validation function which requires an ssh key to be entered if that auth type is selected.
         **/
        public function check_ssh_key($str)
        {
            $type = $this->ci->input->post('auth_type');
            if($type != 'password') {
                if(!strlen($str)) {
                    $this->ci->form_validation->set_message('check_ssh_key', 'This field is required for key authentication.');
                    return FALSE;
                }
            }
            return TRUE;
        }

        /**
         * Form validation function which requires an ssh key password to be entered if that auth type is selected.
         **/
        public function check_ssh_key_pw($str)
        {
            $type = $this->ci->input->post('auth_type');
            if($type == 'key_pw') {
                if(!strlen($str)) {
                    $this->ci->form_validation->set_message('check_ssh_key_pw', 'This field is required.');
                    return FALSE;
                }
            }
            return TRUE;
        }

        /**
         * Sanity check files
         **/
        public function check_filename($str)
        {
            $banned_files = array(
                '/etc/passwd',
                '/etc/hosts',
                '/etc/resolv.conf',
                '/etc/network/interfaces',
                '/etc/sysconfig/network',
            );

            // check for path traversal (dot slash)
            $exp = '/(\.\/)/';
            if(preg_match($exp, $str)) {
                $this->ci->form_validation->set_message('check_filename', 'The filename must be absolute.');
                return FALSE;
            }

            // check for a generally acceptable file
            $exp = '/^\/([^\0]+\/)+([^\0]+)$/';
            if(!preg_match($exp, $str)) {
                $this->ci->form_validation->set_message('check_filename', 'The filename was invalid. Please ensure you have entered a valid, absolute UNIX filename and path.');
                return FALSE;
            }
                 
            // check against banned list
            foreach($banned_files as $file) {
                if($file == $str) {
                    $this->ci->form_validation->set_message('check_filename', 'Uploading the file specified is not allowed.');
                    return FALSE;
                }
            }
            return TRUE;
        }

        /**
         * Get a friendly label for the UI and logs
         **/
        public function get_label()
        {
            if($params = $this->get_params()) {
                if(array_key_exists('hostname', $params)) {
                    return sprintf('Upload local file \'%s\' to remote file \'%s\' on host \'%s\' via %s', $params['local_file'], $params['remote_file'], $params['hostname'],strtoupper($params['ftp_type']));
                }
            }
            return parent::get_label();
        }
       
        /**
         * Make an SSH connection and run a remote command.
         **/
        public function run($context)
        {
            $this->set_context($context);
            if(!file_exists($this->get_param('local_file'))) {
                $this->log('error', 'Local file '. $this->get_param('local_file') .' does not exist.');
                $this->set_result(1);
                return $context;
            }
            switch($this->get_param('ftp_type')) {

                case "ftp":
                    $this->log('info', 'Attempting FTP connection to '. $this->get_param('hostname'));
                    if(!$conn = ftp_connect($this->get_param('hostname'))) {
                        $this->log('error', 'The FTP connection failed.');
                        $this->set_result(1);
                        return $context;
                    }
                    if ($this->get_param('passive')) {
                        ftp_pasv($conn, true);
                    } else {
                        ftp_pasv($conn, false);
                    }
                    if(!$login = ftp_login($conn, $this->get_param('username'), $this->get_param('password'))) {
                        $this->log('error', 'FTP authentication failed.');
                        ftp_close($conn);
                        $this->set_result(1);
                        return $context;
                    }
                    if(!$put_result = ftp_put($conn, $this->get_param('remote_file'), $this->get_param('local_file'), FTP_BINARY)) {
                        $this->log('error', 'File upload failed.');
                        ftp_close($conn);
                        $this->set_result(1);
                        return $context;
                    } else {
                        $this->log('info', 'The file was uploaded successfully.');
                        $this->set_result(0);
                        return $context;
                    }
                    break;

                case "sftp":
                    $this->log('info', 'Attempting SFTP connection to '. $this->get_param('hostname'));
                    $sftp = new Net_SFTP($this->get_param('hostname'));
                    // make this configurable in the ui?
                    $sftp->setTimeout(60);
                    // authenticate
                    switch($this->get_param('auth_type')) {
                        case "password":
                            $this->log('info', 'Using password authentication.');
                            if(!$sftp->login($this->get_param('username'), $this->get_param('password'))) {
                                $this->log('error', 'Password authentication failed.');
                                $this->set_result(1);
                                return $context;
                            }
                            break;
                        case "key":
                        case "key_pw":
                            $key = new Crypt_RSA();
                            if($this->get_param('auth_type') == 'key_pw') {
                                $this->log('info', 'Using password-protected RSA key authentication.');
                                $key->setPassword($this->get_param('ssh_key_pw'));
                            } else {
                                $this->log('info', 'Using RSA key authentication.');
                            }
                            $key->loadKey($this->get_param('ssh_key'));
                            if(!$ssh->login($this->get_param('username'), $key)) {
                                $this->log('error', 'SSH key authentication failed.');
                                $this->set_result(1);
                                return $context;
                            }
                            break;
                    }

                    if(!$sftp->put($this->get_param('remote_file'), $this->get_param('local_file'), NET_SFTP_LOCAL_FILE)) {
                        $this->log('error', 'File upload failed.');
                        $this->set_result(1);
                        return $context;
                    } else {
                        $this->log('info', 'The file was uploaded successfully.');
                        $this->set_result(0);
                        return $context; 
                    }
                    break;
            }
                
        }
    }
